//////////////////////////////////////////////////////////////////////////////
// nitrogen.cpp
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
// Copyright (c) 2006, 2007 Riccardo Iaconelli <riccardo@kde.org>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include "nitrogen.h"
#include "nitrogen.moc"
#include "nitrogenclient.h"
#include "nitrogenshadowcache.h"

#include <cassert>
#include <KDebug>
#include <KConfigGroup>
#include <KStandardDirs>
#include <kdeversion.h>
#include <kwindowinfo.h>
#include <QApplication>
#include <QPainter>

extern "C"
{
  KDE_EXPORT KDecorationFactory* create_factory()
  { return new Nitrogen::NitrogenFactory(); }
}

namespace Nitrogen
{

  K_GLOBAL_STATIC_WITH_ARGS(OxygenHelper, globalHelper, ("oxygenDeco"))
  K_GLOBAL_STATIC_WITH_ARGS(NitrogenShadowCache, globalShadowCache, (maxAnimationIndex))

  //___________________________________________
  OxygenHelper *nitrogenHelper()
  { return globalHelper; }

  //___________________________________________
  NitrogenShadowCache *nitrogenShadowCache()
  {
    NitrogenShadowCache* cache = globalShadowCache;
    return cache;
  }

  // initialize static members
  bool NitrogenFactory::initialized_ = false;
  NitrogenConfiguration NitrogenFactory::defaultConfiguration_;
  NitrogenExceptionList NitrogenFactory::exceptions_;

  //___________________________________________________
  NitrogenFactory::NitrogenFactory()
  {
    kDebug( 1212 ) << "Version: " << APP_VERSION << endl;
    readConfig();
    setInitialized( true );
  }

  //___________________________________________________
  NitrogenFactory::~NitrogenFactory()
  { setInitialized( false ); }

  //___________________________________________________
  KDecoration* NitrogenFactory::createDecoration(KDecorationBridge* bridge )
  {
    NitrogenClient* client( new NitrogenClient( bridge, this ) );
    connect( this, SIGNAL( configurationChanged() ), client, SLOT( resetConfiguration() ) );
    return client->decoration();
  }

  //___________________________________________________
  bool NitrogenFactory::reset(unsigned long changed)
  {

    kDebug( 1212 ) << endl;

    // read in the configuration
    setInitialized( false );
    bool configuration_changed = readConfig();
    setInitialized( true );

    if( configuration_changed || (changed & (SettingDecoration | SettingButtons | SettingBorder)) )
    {

      kDebug(1212) << "configuration_changed: " << configuration_changed << endl;
      emit configurationChanged();
      return true;

    } else {

      if( changed & SettingColors ) nitrogenShadowCache()->invalidateCaches();
      resetDecorations(changed);
      return false;

    }

  }

  //___________________________________________________
  bool NitrogenFactory::readConfig()
  {

    // create a config object
    KConfig config("nitrogenrc");

    QString file = KStandardDirs::locate( "config", "nitrogenrc" );
    kDebug( 1212 ) << " Configuration file name: " << file << endl;

    KConfigGroup group( config.group("Windeco") );
    NitrogenConfiguration configuration( group );
    bool changed = !( configuration == defaultConfiguration() );

    // check configurations
    changed |= (NitrogenConfiguration::useCompiz() != NitrogenConfiguration::checkUseCompiz());

    // read exceptionsreadConfig
    NitrogenExceptionList exceptions( config );
    if( !( exceptions == exceptions_ ) )
    {
      exceptions_ = exceptions;
      changed = true;
    }


    if( changed )
    {

      nitrogenHelper()->invalidateCaches();
      setDefaultConfiguration( configuration );
      return true;

    } else return false;

  }

  //_________________________________________________________________
  bool NitrogenFactory::supports( Ability ability ) const
  {
    switch( ability )
    {

      // announce
      case AbilityAnnounceButtons:
      case AbilityAnnounceColors:

      // buttons
      case AbilityButtonMenu:
      case AbilityButtonHelp:
      case AbilityButtonMinimize:
      case AbilityButtonMaximize:
      case AbilityButtonClose:
      case AbilityButtonOnAllDesktops:
      case AbilityButtonAboveOthers:
      case AbilityButtonBelowOthers:
      case AbilityButtonSpacer:
      case AbilityButtonShade:

      // compositing
      #if KDE_IS_VERSION(4,2,92)
      case AbilityProvidesShadow: // TODO: UI option to use default shadows instead
      case AbilityUsesAlphaChannel:
      return !NitrogenConfiguration::useCompiz();

      #else
      case AbilityCompositingShadow:
      return !NitrogenConfiguration::useCompiz();
      #endif

      // no colors supported at this time
      default:
      return false;
    };
  }


  #if !KDE_IS_VERSION(4,2,92)

  //////////////////////////////////////////////////////////////////////////////
  // Shadows (kde4.2 version)

  //_________________________________________________________________
  QList< QList<QImage> > NitrogenFactory::shadowTextures()
  {

    QList< QList<QImage> > textureLists;

    // in compiz mode, no textures are used
    if( NitrogenConfiguration::useCompiz() )
    { return textureLists; }

    textureLists.append( makeTextures( Active ) );
    textureLists.append( makeTextures( Inactive ) );
    return textureLists;

  }

  //_________________________________________________________________
  QList<QImage> NitrogenFactory::makeTextures( NitrogenFactory::Activity state ) const
  {

    // palette
    QPalette palette = qApp->palette();
    palette.setCurrentColorGroup(QPalette::Active);

    NitrogenShadowCache::Key key;
    key.active = (state == Active );
    key.isShade = false;
    switch( defaultConfiguration().frameBorder() )
    {
      case NitrogenConfiguration::BorderNone: key.frameBorder = NitrogenShadowCache::Key::BorderNone; break;
      case NitrogenConfiguration::BorderNoSide:  key.frameBorder = NitrogenShadowCache::Key::BorderNoSide; break;
      default:  key.frameBorder = NitrogenShadowCache::Key::BorderAny; break;
    }

    if( ( defaultConfiguration().drawTitleOutline() || !defaultConfiguration().overwriteColors() ) && defaultConfiguration().frameBorder() == NitrogenConfiguration::BorderNone )
    {

      QColor inactive = palette.color( QPalette::Window );
      QColor active = KDecoration::options()->color(ColorTitleBar, true);

      key.useOxygenShadows = defaultConfiguration().useOxygenShadows();
      key.hasTitleOutline = true;

      //---------------------------------------------------------------
      // Create new glow/shadow tiles
      qreal shadowSize( nitrogenShadowCache()->shadowSize() );
      QPixmap shadow = QPixmap( shadowSize*2, shadowSize*2 );
      shadow.fill( Qt::transparent );

      QPainter p( &shadow );
      p.setRenderHint( QPainter::Antialiasing );

      QPixmap shadowTop( nitrogenShadowCache()->simpleShadowPixmap( active, key ) );
      QRect topRect( shadow.rect() );
      topRect.setBottom( shadowSize-1 );
      p.setClipRect( topRect );
      p.drawPixmap( QPointF( 0, 0 ), shadowTop );

      QPixmap shadowBottom( nitrogenShadowCache()->simpleShadowPixmap( inactive, key ) );
      QRect bottomRect( shadow.rect() );
      bottomRect.setTop( shadowSize );
      p.setClipRect( bottomRect );
      p.drawPixmap( QPointF( 0, 0 ), shadowBottom );
      p.end();

      return makeTextures( shadow );

    } else {

      key.useOxygenShadows = defaultConfiguration().useOxygenShadows();
      key.hasTitleOutline = defaultConfiguration().drawTitleOutline() || !defaultConfiguration().overwriteColors();

      QColor color = palette.color( QPalette::Window );
      if( key.hasTitleOutline ) color = KDecoration::options()->color(ColorTitleBar, key.active );

      return makeTextures( nitrogenShadowCache()->simpleShadowPixmap( color, key )  );

    }

  }

  //______________________________________________________
  QImage NitrogenFactory::makeTexture( const QPixmap& pixmap, const QRect& rect ) const
  {
    QPixmap dump( rect.size() );
    dump.fill( Qt::transparent );
    QPainter painter( &dump );
    painter.drawPixmap( QPoint(0, 0), pixmap, rect );
    painter.end();
    return dump.toImage();
  }

  //___________________________________________________________
  QList<QImage> NitrogenFactory::makeTextures( const QPixmap& shadow ) const
  {
    QList<QImage> textures;
    int w = shadow.width() / 2;
    int h = shadow.height() / 2;
    textures.append( makeTexture( shadow, QRect( 0, h+1, w, h ) ) );
    textures.append( makeTexture( shadow, QRect( w, h+1, 1, h ) ) );
    textures.append( makeTexture( shadow, QRect( w+1, h+1, w, h ) ) );
    textures.append( makeTexture( shadow, QRect( 0, h, w, 1 ) ) );
    textures.append( makeTexture( shadow, QRect( w+1, h, w, 1 ) ) );
    textures.append( makeTexture( shadow, QRect( 0, 0, w, h ) ) );
    textures.append( makeTexture( shadow, QRect( w, 0, 1, h ) ) );
    textures.append( makeTexture( shadow, QRect( w+1, 0, w, h ) ) );
    return textures;
  }

  //_________________________________________________________________
  int NitrogenFactory::shadowTextureList( ShadowType type ) const
  {
    switch( type ) {

      case ShadowBorderedInactive: return 1;
      default: return 0;

    }
    abort(); // Should never be reached
  }

  //_________________________________________________________________
  QList<QRect> NitrogenFactory::shadowQuads( ShadowType, QSize ) const
  { return QList<QRect>(); }

  //____________________________________________________________________
  double NitrogenFactory::shadowOpacity( ShadowType ) const
  { return 1.0; }
  #endif

  //____________________________________________________________________
  NitrogenConfiguration NitrogenFactory::configuration( const NitrogenClient& client )
  {

    if( exceptions_.empty() )
    { return defaultConfiguration(); }

    KWindowInfo info( KWindowInfo( client.windowId(), NET::WMName, NET::WM2WindowClass|NET::WM2ClientMachine ) );

    QString window_title;
    QString class_name;
    QString machine_name;
    for( NitrogenExceptionList::const_iterator iter = exceptions_.constBegin(); iter != exceptions_.constEnd(); iter++ )
    {

      // discard disabled exceptions
      if( !iter->enabled() ) continue;

      /*
      decide which value is to be compared
      to the regular expression, based on exception type
      */
      QString value;
      if( iter->type() == NitrogenException::WindowTitle )
      {

        value = window_title.isEmpty() ? (window_title = info.name() ):window_title;

      } else if( iter->type() == NitrogenException::WindowClassName ) {

        // retrieve class name
        if( class_name.isEmpty() )
        {
          QString window_class_name( info.windowClassName() );
          QString window_class( info.windowClassClass() );
          class_name = window_class_name + " " + window_class;
        }

        value = class_name;

      } else if( iter->type() == NitrogenException::WindowMachine ) {

        value = machine_name.isEmpty() ? ( machine_name = info.clientMachine() ):machine_name;

      }

      if( iter->regExp().indexIn( value ) < 0 ) continue;
      NitrogenConfiguration configuration( defaultConfiguration() );

      // propagate all features found in mask to the output configuration
      if( iter->mask() & NitrogenException::FrameBorder ) configuration.setFrameBorder( iter->frameBorder() );
      if( iter->mask() & NitrogenException::BlendColor ) configuration.setBlendColor( iter->blendColor() );
      if( iter->mask() & NitrogenException::SeparatorType ) configuration.setSeparatorType( iter->separatorType() );
      if( iter->mask() & NitrogenException::SizeGripMode ) configuration.setSizeGripMode( iter->sizeGripMode() );
      if( iter->mask() & NitrogenException::ShowStripes ) configuration.setShowStripes( iter->showStripes() );
      if( iter->mask() & NitrogenException::OverwriteColors ) configuration.setOverwriteColors( iter->overwriteColors() );

      return configuration;

    }

    return defaultConfiguration();

  }

} //namespace Nitrogen
