#ifndef nitrogenclient_h
#define nitrogenclient_h

//////////////////////////////////////////////////////////////////////////////
// nitrogenclient.h
// -------------------
//
// Copyright (c) 2009 Hugo Pereira Da Costa <hugo.pereira@free.fr>
// Copyright (c) 2003, 2004 David Johnson <david@usermode.org>
// Copyright (c) 2006, 2007 Riccardo Iaconelli <ruphy@fsfe.org>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//////////////////////////////////////////////////////////////////////////////

#include <kcommondecoration.h>
#include <QtCore/QTimeLine>

#include "kdeversion.h"
#include "nitrogenconfiguration.h"
#include "lib/helper.h"
#include "lib/tileset.h"


namespace Nitrogen
{

    class NitrogenSizeGrip;

    class NitrogenClient : public KCommonDecorationUnstable
    {

        Q_OBJECT

        public:

        //! constructor
        NitrogenClient(KDecorationBridge *b, KDecorationFactory *f);

        //! destructor
        virtual ~NitrogenClient();

        virtual QString visibleName() const;
        virtual KCommonDecorationButton *createButton(::ButtonType type);
        virtual bool decorationBehaviour(DecorationBehaviour behaviour) const;

        //! true if window is maximized
        virtual bool isMaximized( void ) const
        { return maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();  }

        //! return true if timeLine is running
        bool timeLineIsRunning( void ) const
        { return timeLine_.state() == QTimeLine::Running; }

        //! true when separator is to be drawn
        virtual bool drawSeparator( void ) const
        {
          return
            ( ( ( isActive() || timeLineIsRunning() ) && configuration().separatorType() == NitrogenConfiguration::SeparatorActive) ||
            configuration().separatorType() == NitrogenConfiguration::SeparatorAlways ) &&
            !configuration().drawTitleOutline();
        }

        //! true when oxygen 'glow' shadow is to be drawn for active window
        virtual bool useOxygenShadows( void ) const
        { return isActive() && configuration().useOxygenShadows(); }

        virtual bool showStripes( void ) const
        {
          return ( isActive() || timeLineIsRunning() ) && configuration().showStripes() && !configuration().drawTitleOutline();
        }

        //! dimensions
        virtual int layoutMetric(LayoutMetric lm, bool respectWindowState = true, const KCommonDecorationButton * = 0) const;

        //! window shape
        virtual void updateWindowShape();

        //! initialization
        virtual void init();

        #if !KDE_IS_VERSION(4,2,92)
        virtual QList<QRect> shadowQuads( ShadowType type ) const;
        virtual double shadowOpacity( ShadowType type ) const;
        #endif

        //! return associated configuration
        NitrogenConfiguration configuration( void ) const;

        //! return timeLine
        const QTimeLine& timeLine( void ) const
        { return timeLine_; }

        //! helper class
        OxygenHelper& helper( void ) const
        { return helper_; }

        //! return animation opacity
        qreal opacity( void ) const
        { return qreal( timeLine_.currentFrame() )/qreal( timeLine_.endFrame() ); }

        //! palette background
        QPalette backgroundPalette( const QWidget*, QPalette ) const;

        //! background color
        QColor backgroundColor( const QWidget* widget, QPalette palette ) const
        { return backgroundColor( widget, palette, isActive() ); }

        //! background color
        QColor backgroundColor( const QWidget* widget, QPalette palette, bool active ) const;

        //! window background
        virtual void renderWindowBackground( QPainter*, const QRect&, const QWidget*, const QPalette& ) const;

        //! window border
        // this draws a "blue" border around active window
        virtual void renderWindowBorder( QPainter*, const QRect&, const QWidget*, const QPalette& ) const;

        //! separator
        virtual void renderSeparator( QPainter*, const QRect&, const QWidget*, const QColor& ) const;

        //! title outline
        virtual void renderTitleOutline( QPainter*, const QRect&, const QPalette& ) const;

        //! title outline
        virtual void renderTitleText( QPainter*, const QRect&, Qt::Alignment, QColor ) const;

        //! triggered when window activity is changed
        virtual void activeChange();

        //! triggered when window shade is changed
        virtual void maximizeChange();

        //! triggered when window shade is changed
        virtual void shadeChange();

        //! triggered when window shade is changed
        virtual void captionChange();

        signals:

        //! emmited when update is needed
        void dirty( void );

        //! emmited when configuration is changed
        void configurationChanged();

        public slots:

        //! reset configuration
        void resetConfiguration( void );

        protected:

        //! paint
        void paintEvent( QPaintEvent* );

        //! title timeline
        bool titleTimeLineIsRunning( void ) const
        { return titleTimeLine_.state() == QTimeLine::Running; }

        //! title opacity
        qreal titleOpacity( void ) const
        { return qreal( titleTimeLine_.currentFrame() )/qreal( titleTimeLine_.endFrame() ); }

        //! old caption if any
        const QString& oldCaption( void ) const
        { return oldCaption_; }

        //! old caption
        void setOldCaption( const QString& value )
        { oldCaption_ = value; }

        //! draw
        void drawStripes(QPainter*, QPalette&, const int, const int, const int);

        //! return true when activity change are animated
        bool animateActiveChange( void ) const
        { return configuration().useAnimations() && !isPreview(); }

        //! return true when activity change are animated
        bool animateTitleChange( void ) const
        {
          return
            configuration().useAnimations() &&
            !configuration().drawTitleOutline() &&
            !configuration().showStripes() &&
            !isPreview();
        }

        //! calculate mask
        QRegion calcMask( void ) const;

        //! text color
        QColor titlebarTextColor(const QPalette&);

        //! text color
        QColor titlebarTextColor(const QPalette&, bool active);

        //!@name size grip
        //@{

        //! create size grip
        void createSizeGrip( void );

        //! delete size grip
        void deleteSizeGrip( void );

        // size grip
        bool hasSizeGrip( void ) const
        { return (bool)size_grip_; }

        //! size grip
        NitrogenSizeGrip& sizeGrip( void ) const
        { return *size_grip_; }

        //@}

        protected slots:

        //! update old caption with current
        void updateOldCaption( void )
        { setOldCaption( caption() ); }

        private:

        //! configuration
        NitrogenConfiguration configuration_;

        //! used to invalidate color cache
        bool colorCacheInvalid_;

        //! stored color
        QColor cachedTitlebarTextColor_;

        //! size grip widget
        NitrogenSizeGrip* size_grip_;

        //! animation timeLine
        QTimeLine timeLine_;

        //! title animation timeLine
        QTimeLine titleTimeLine_;

        //! old caption
        QString oldCaption_;

        //! helper
        OxygenHelper& helper_;

        //! true when initialized
        bool initialized_;

    };


} // namespace Nitrogen

#endif // EXAMPLECLIENT_H
